import pytest
from django.template import Context, Template

from .forms import ContactForm

icon_test = (
    '<svg class="test" fill="currentColor" viewbox="0 0 20 20"'
    ' xmlns="http://www.w3.org/2000/svg">\n<path d="M6.28 5.22a.75.75 0'
    " 00-1.06 1.06L8.94 10l-3.72 3.72a.75.75 0 101.06 1.06L10 11.06l3.72"
    " 3.72a.75.75 0 101.06-1.06L11.06 10l3.72-3.72a.75.75 0 00-1.06-1.06L10"
    ' 8.94 6.28 5.22z"></path>\n</svg>'
)
icon_aria = (
    '<svg aria-hidden="true" class="test" fill="currentColor" viewbox="0 0 20'
    ' 20" xmlns="http://www.w3.org/2000/svg">\n<path d="M6.28 5.22a.75.75 0'
    " 00-1.06 1.06L8.94 10l-3.72 3.72a.75.75 0 101.06 1.06L10 11.06l3.72"
    " 3.72a.75.75 0 101.06-1.06L11.06 10l3.72-3.72a.75.75 0 00-1.06-1.06L10"
    ' 8.94 6.28 5.22z"></path>\n</svg>'
)


@pytest.mark.parametrize(
    "template, html",
    [
        (
            (
                '{% load fragments %}{% icon name="x_mark_mini" css="test" pre_text="on'
                ' the left" %}'
            ),
            f"<span>on the left</span>{icon_test}",
        ),
        (
            (
                '{% load fragments %}{% icon name="x_mark_mini" css="test"'
                ' post_text="on the right" %}'
            ),
            f"{icon_test}<span>on the right</span>",
        ),
        (
            (
                '{% load fragments %}{% icon name="x_mark_mini" css="test"'
                ' parent_class="cover" post_text="on the right" %}'
            ),
            f'<span class="cover">{icon_test}<span>on the right</span></span>',
        ),
        (
            (
                '{% load fragments %}{% icon name="x_mark_mini" css="test"'
                ' pre_text="Close menu" pre_class="sr-only" aria_hidden="true" %}'
            ),
            f'<span class="sr-only">Close menu</span>{icon_aria}',
        ),
    ],
)
def test_icon_fragments(template, html):
    assert Template(template).render(context=Context()) == html


@pytest.mark.parametrize(
    "template, html",
    [
        (
            """{% load fragments %}{% whitespaceless %}
            <p class="  test
                        test2
                        test3  ">
                <a href="foo/">Foo</a>
            </p>{% endwhitespaceless %}""",
            '<p class="test test2 test3"><a href="foo/">Foo</a></p>',
        ),
    ],  # noqa: E501; also from [Will Gordon](https://stackoverflow.com/users/6758654/will-gordon). See [answer](https://stackoverflow.com/a/72942459)
)
def test_whitespace(template, html):
    assert Template(template).render(context=Context()) == html


def test_boundfield_with_inline_validation():
    template = """{% load fragments %}{% spaceless %}{% hput form.email validate="/this-is-an-endpoint" %}{% endspaceless %}"""
    context = Context({"form": ContactForm()})
    html = (
        '<div id="hput_id_email" hx-select="#hput_id_email"'
        ' hx-post="/this-is-an-endpoint" hx-trigger="blur from:find input"'
        ' hx-target="#hput_id_email" hx-swap="outerHTML" class="h"'
        ' data-widget="email"><label for="id_email">Email</label><input type="email"'
        ' name="email" required id="id_email"><small>Testable form</small></div>'
    )
    assert Template(template).render(context=context) == html



def filter_attrs(key: str, d: dict, unprefix: bool = False) -> dict[str, str]:
    """Filter `k`, `v` from `d` based on keys prefixed with `<key>_`. Based on this result, rename or replace the key,
    depending on the `unprefix` flag.

    This enables a shortcut for gathering all `<key>-`* attributes found in the dict `d` and parse them properly
    before inserting them into html tags.

    Examples:
        >>> res = filter_attrs(key="aria", d={"aria_hidden":"true"})
        >>> res['aria-hidden'] == "true"
        True
        >>> res_hx = filter_attrs(key="hx", d={"hx_get":"https://test.html",  "hx_target":"body"})
        >>> res_hx['hx-get'] == "https://test.html"
        True
        >>> res_hx['hx-target'] == "body"
        True
        >>> res_dt = filter_attrs(key="data", d={"data_site_good":"https://test.html"})
        >>> res_dt['data-site-good'] == "https://test.html"
        True
        >>> parent_res = filter_attrs(key="parent", d={"non-a-parent": "test", "parent_class":"flex items-center", "parent_title": "I should be centered"}, unprefix=True)
        >>> "parent_class" in parent_res
        False
        >>> "class" in parent_res
        True
        >>> "title" in parent_res
        True
        >>> "non-a-parent" in parent_res
        False
        >>> pre_res = filter_attrs(key="pre", d={"pre_class":"sr-only"}, unprefix=True)
        >>> "class" in pre_res
        True
        >>> res_btn = filter_attrs(key="btn", d={"btn_name":"i-am-button", "btn_id": "btn-1"}, unprefix=True)
        >>> res_btn["name"] == "i-am-button"
        True
        >>> res_btn["id"] == "btn-1"
        True
        >>> res_a= filter_attrs(key="a", d={"a_href":"#", "a_target": "_self"}, unprefix=True)
        >>> res_a["href"] == "#"
        True
        >>> res_a["target"] == "_self"
        True

    Args:
        d (dict): Values from a template tag.

    Returns:
        dict[str, str]: dict to be used for a html tag's aria-* attributes.
    """  # noqa: E501
    return {
        (k.removeprefix(f"{key}_") if unprefix else k.replace("_", "-")): v
        for k, v in d.items()
        if k.startswith(f"{key}_")
    }



from django.urls import path
from django.urls.resolvers import URLPattern

from .views import AboutPage, ContactFormView, HomePage, send_msg

app_name = "test_fragments"
urlpatterns: list[URLPattern] = [
    path(route="send_msg/", view=send_msg, name="send_msg"),
    path(route="contact/", view=ContactFormView.as_view(), name="contact"),
    path(route="about/", view=AboutPage.as_view(), name="about"),
    path(route="", view=HomePage.as_view(), name="test_page"),
]



import os
import sys
from pathlib import Path

import django
from django.http.request import HttpRequest


def is_htmx(request: HttpRequest) -> bool:
    """Determines whether or not the request should be handled differently
    because of the presence of the `HTTP_HX_REQUEST` header.

    Args:
        request (HttpRequest): The Django request object received from the view

    Returns:
        bool: Whether or not `HTTP_HX_REQUEST` exists.
    """
    return True if request.META.get("HTTP_HX_REQUEST") else False


def prep_nb(base_dir_path: Path = Path().cwd()):
    sys.path.insert(0, str(base_dir_path))
    os.environ.setdefault(
        "DJANGO_SETTINGS_MODULE", "config.settings"
    )  # Access django imports
    os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = (
        "true"  # Allow qs async filtering in a cell
    )
    django.setup()  # This is for setting up django
